/*
This file is part of ObjetcsOS.

ObjetcsOS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

ObjetcsOS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ObjectsOS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "system.h"
#include "floppy.h"
#include "data.h"
#include "screen.h"

void *sectorbuffer = (void*)0xF0000;           /*Puffer fr 10 Sektoren*/
void *kernelmem = (void*)0xE0000;              /*Platz fr den Kernel*/
MOTOR_STATUS motor = OFF;

void initFloppy()
{
        motorON();
        WaitTicks(2);
        resetFloppy();
        motorOFF();
}

void motorOFF() { outportb(0x3F2, 0); motor = ON; }
void motorON() { outportb(0x3F2, 0x1C); motor = OFF;}

void WaitTicks(DWORD ticks)
{
        extern DWORD TickCounter;
        DWORD i;
        
        i = TickCounter;
        while(ticks <= TickCounter-i) asm("nop");
}

void WaitFdc()
{
        while((inportb(0x3F4) & 0x80) != 0x80);
}

void WaitFdcInt()
{
        while(!getIrqFired(6));
        deleteIrqFired(6);
}

void sendFloppyByte(BYTE b)
{
        BYTE testbyte;

        do {
                testbyte = inportb(0x03F4);
        } while((testbyte & 0x80) != 0x80);
        WaitTicks(1);
        outportb(0x3F5, b);
}

BYTE receiveFloppyByte()
{
        BYTE testbyte;
        
        do {
                testbyte = inportb(0x03F4);
        } while((testbyte & 0x80) != 0x80);
        WaitTicks(1);
        return inportb(0x3F5);
}

void recalibrate()
{
        BYTE tb;
        
        do {
                sendFloppyByte(0x07);
                sendFloppyByte(0x00);
                tb = receiveFloppyByte();
        } while((tb & 0x80) != 0x80);
}

BYTE seek(BYTE head, BYTE track)
{
        BYTE dummy;
        while(track >= 80) track-=80;

        do {
                deleteIrqFired(6);
                sendFloppyByte(0x0F);
                sendFloppyByte(head << 2);
                sendFloppyByte(track);
                WaitFdcInt();
                sendFloppyByte(0x08);
                receiveFloppyByte();
                dummy = receiveFloppyByte();

        } while((receiveFloppyByte() & 0x80) == 0x80);
        return dummy;
}

void resetFloppy()
{
        deleteIrqFired(6);
        outportb(FDC_DOR,0);
        outportb(FDC_DRS, 0);
        outportb(FDC_DOR,0x0c);
        WaitFdcInt();


        sendFloppyByte(CMD_SPECIFY);
        sendFloppyByte(0xDF);
        sendFloppyByte(0x02);

        recalibrate();
        seek(0, 0);
}

void FloppyIsr()
{
        setIrqFired(6);
        outportb(0x20, 0x20);
}

void startDMA(BYTE mode, void* buffer, WORD len)
{
        BYTE page; WORD offset;
        offset = (DWORD)buffer & 0xFFFF;
        page = (BYTE)((DWORD)buffer >> 16);
        
        mode |= DMACHANNEL;
        len--;
        
        disableInts();
        outportb(MASKREG, 0x04 | DMACHANNEL);
        outportb(CLEARREG, 0);
        outportb(MODEREG, mode);

        outportb(ADDRPORT, LO(offset));
        outportb(ADDRPORT, HI(offset));
        outportb(PAGEPORT, page);
        
        outportb(COUNTPORT, LO(len));
        outportb(COUNTPORT, HI(len));
        outportb(MASKREG, DMACHANNEL);
        enableInts();
}

void stopDMA()
{
        outportb(MASKREG, 0x04 | DMACHANNEL);
        outportb(CLEARREG, 0);
        outportb(MASKREG, DMACHANNEL);
}

BOOL readSector(WORD sector, void* buffer)
{
        BYTE h, t, s, tries;
        
        startDMA(0x44, buffer, 512);

        motorON();
        WaitTicks(2);

        deleteIrqFired(6);
        recalibrate();
        WaitFdcInt();

        h = sector / 1440;
        t = (sector % 1440) / 18;
        s = (sector % 1440) % 18;
        
        for(tries = 1; tries <= 3; tries++) {
                sector = seek(h, t);
                if(t == sector) break;
                write("Seeking failed: T="); write(inttostr(t)); write(" S=");
                write(inttostr(s)); write(" returned: "); writeln(inttostr(sector));
                if(tries == 3) {
                        motorOFF();
                        stopDMA();
                        return false;
                }
        }
        
        outportb(FDC_CCR,0);
        
        deleteIrqFired(6);
        sendFloppyByte(0x66); /*single Track, double density, skip deleted data, read*/
        writeln(inttohex(inportb(0x3F4)));
        sendFloppyByte(h << 2);
        sendFloppyByte(t);
        sendFloppyByte(h);
        sendFloppyByte(s);
        sendFloppyByte(2);
        sendFloppyByte(18);
        sendFloppyByte(27);
        sendFloppyByte(0x0);
        WaitFdcInt();
        writeln(inttohex(inportb(0x3F4)));
        h = receiveFloppyByte() & 0xC0;
        write(inttostr(h));simulTab();
        write(inttostr(receiveFloppyByte())); simulTab();
        write(inttostr(receiveFloppyByte())); simulTab();
        write(inttostr(receiveFloppyByte())); simulTab();
        write(inttostr(receiveFloppyByte())); simulTab();
        write(inttostr(receiveFloppyByte())); simulTab();
        writeln(inttostr(receiveFloppyByte())); simulTab();
        
        WaitTicks(3);
        motorOFF();
        stopDMA();
        return (h == 0);
}
