/*
This file is part of ObjetcsOS.

ObjetcsOS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

ObjetcsOS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ObjectsOS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _oos_include_floppy_h
#define _oos_include_floppy_h

#include "types.h"

#define FDC_DOR  (0x3f2)   /* Digital Output Register */
#define FDC_MSR  (0x3f4)   /* Main Status Register (input) */
#define FDC_DRS  (0x3f4)   /* Data Rate Select Register (output) */
#define FDC_DATA (0x3f5)   /* Data Register */
#define FDC_DIR  (0x3f7)   /* Digital Input Register (input) */
#define FDC_CCR  (0x3f7)   /* Configuration Control Register (output) */

#define CMD_SPECIFY 3

#define DMACHANNEL      2
#define MASKREG         0x0A
#define MODEREG         0x0B
#define CLEARREG        0x0C
#define PAGEPORT        0x81
#define ADDRPORT        0x04
#define COUNTPORT       0x05

#define LO(x)   (x & 0xFF)
#define HI(x)   ((x & 0xFF00) >> 8)

typedef struct {
        BYTE oem_id[8];
        WORD bytes_per_sector;
        DWORD sectors_per_cluster;
        WORD reserved_sectors;
        BYTE fats;      /*Anzahl der FATs*/
        WORD rootdir_entries;
        WORD logic_sectors;
        BYTE media_id;
        WORD sectors_per_fat;
        WORD sectors_per_track;
        DWORD hidden_sectors;
        DWORD logic_sectors_long;
        BYTE drive_number;
        BYTE reserved;
        BYTE boot_signature;
        DWORD serial;
        BYTE label[11];
        BYTE file_system_id[8];
} FAT_HEADER;

void initFloppy();
void motorOFF();
void motorON();
void WaitTicks(DWORD ticks);
void WaitFdc();
void WaitFdcInt();
void sendFloppyByte(BYTE b);
BYTE receiveFloppyByte();
void recalibrate();
BYTE seek(BYTE head, BYTE track);
void resetFloppy();
void FloppyIsr();

void startDMA(BYTE mode, void* buffer, WORD len);
void stopDMA();
BOOL readSector(WORD sector, void* buffer);

#endif
