;       This file is part of ObjetcsOS.

;       ObjetcsOS is free software; you can redistribute it and/or modify
;       it under the terms of the GNU General Public License as published by
;       the Free Software Foundation; either version 2 of the License, or
;       (at your option) any later version.

;       ObjetcsOS is distributed in the hope that it will be useful,
;       but WITHOUT ANY WARRANTY; without even the implied warranty of
;       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;       GNU General Public License for more details.

;       You should have received a copy of the GNU General Public License
;       along with ObjectsOS; if not, write to the Free Software
;       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

[BITS 16]
jmp start

;-------------------------------------------------------------------- macros

%macro	descriptor	6
	DW	%1
	DW	%2
	DB	%3
	DB	%4
	DB	%5
	DB	%6
%endmacro



%macro	setbase	3
	mov	ax,	%2
	and	eax,	0xFFFF
	shl	eax,	0x04
	add	ax,	%3

	mov	word[_gdtstart+%1*8+2],	ax
	shr	eax,	16
	mov	byte[_gdtstart+%1*8+4],	al
%endmacro

;-------------------------------------------------------------------- GDT

gdtr:	DW	gdtend-_gdtstart-1		;Limit
gbase:	DD	0x00000000			;Start

[global _gdtstart]
;                align           8
_gdtstart:	descriptor	0x0000, 0x0000, 0x00, 00000000b, 00000000b, 0x00	;Nulldeskriptor
		descriptor	0xFFFF, 0x0000, 0x00, 10011010b, 11001111b, 0x00	;Codesegment
		descriptor	0xFFFF, 0x0000, 0x00, 10010010b, 11001111b, 0x00	;Datensegment
		descriptor	0x0F9F, 0x8000, 0x0B, 10010010b, 00000000b, 0x00	;Videoram-Segment
		descriptor	0x2000, 0x9000, 0x0B, 10010010b, 00000000b, 0x00	;Stacksegment
                descriptor      0xFFFF, 0x0000, 0x00, 10010010b, 11001111b, 0x00        ;linear
gdtend:

idtr:   DW      idtend - _idtstart-1
ibase:  DD      0x0

[global _idtstart]
_idtstart:
        TIMES 256*8 DB 0
idtend:

nilsel	                      equ	0000000000000000b
codesel	                      equ	0000000000001000b
datasel	                      equ	0000000000010000b
vidsel                        equ	0000000000011000b
stacksel                     equ	0000000000100000b

;-------------------------------------------------------------------- data
;-------------------------------------------------------------------- code
[BITS 16]
[global start]
start:
		push		cs
		pop		ds					;DS = CS
  
		xor		eax,	eax
		mov		ax,	ds
		shl		eax,	4				;eax * 16
		add		eax,	_gdtstart
		mov		[ds:gbase],	eax			;gdtbase = cs*16+offset gdtstart
                sub             eax, _gdtstart
                add             eax, _idtstart
                mov		[ds:ibase],	eax

		setbase	1, cs, 0
		setbase	2, ds, 0

		cli

o32		lgdt		[gdtr]
o32		lidt		[idtr]

		mov		eax,	cr0
		or		eax,	1
		mov		cr0,	eax

		jmp		codesel:postswitch
[bits 32]
postswitch:
		mov		ax,	datasel
		mov		ds,	ax
		mov		es,	ax
		mov		ax,	vidsel
		mov		gs,	ax
		mov		ax,	stacksel
		mov		ss,	ax
		mov		esp,	0x2000

extern _main
        call   _main
spin:
 	jmp		spin					; loop
;-------------------------------------------------------------------- end

[global _enableA20]
_enableA20:
	call    a20wait
	mov     al,0xAD
	out     0x64,al

	call    a20wait
	mov     al,0xD0
	out     0x64,al

	call    a20wait2
	in      al,0x60
	push    eax

	call    a20wait
	mov     al,0xD1
	out     0x64,al

	call    a20wait
	pop     eax
	or      al,2
	out     0x60,al

        call    a20wait
	mov     al,0xAE
	out     0x64,al

	call    a20wait
	ret

a20wait:
.l0:	mov     ecx,65536
.l1:	in      al,0x64
	test    al,2
	jz      .l2
	loop    .l1
	jmp     .l0
.l2:	ret


a20wait2:
.l0:	mov     ecx,65536
.l1:	in      al,0x64
	test    al,1
	jnz     .l2
        loop    .l1
	jmp     .l0
.l2:	ret
