/*
This file is part of ObjetcsOS.

ObjetcsOS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

ObjetcsOS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ObjectsOS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "system.h"
#include "data.h"
#include "screen.h"

DTINFO gdtinfo;                /*Die globale GDTINFO*/
DTINFO idtinfo;
WORD waitForIrqBitmap = 0;
DWORD TickCounter;

extern SYSTEM_INFO system;

unsigned char inportb(WORD port)
{
   unsigned char result;

   asm volatile ("inb %%dx,%%al":"=a" (result):"d" (port));
   return result;
}

void outportb(WORD port, BYTE value)
{
   asm volatile ("outb %%al,%%dx"::"d" (port), "a" (value));
}

void getGDTInfo()
{
        extern void gdtstart;
        gdtinfo.offset = &gdtstart;
        gdtinfo.items = INITIAL_GDT_ITEMS;
        gdtinfo.datasel = SYS_DATA_SEL;
}

/*
void printGDT(unsigned from, unsigned to)
{
        unsigned i;
        DESCRIPTOR *dummy;
        if(from > to) return;
        write("GDT.OFFSET: "); write(inttohex((DWORD)gdtinfo.offset)); simulTab(); write("GDT.ITEMS: "); writeln(inttohex(gdtinfo.items));
        for(i = from; i <= to; i++) {
                dummy = gdtinfo.offset + 8 * i;
                write(inttohex(dummy->size0)); simulTab(); write(inttohex(dummy->base0)); simulTab();
                write(inttohex(dummy->base1)); simulTab(); write(inttohex(dummy->access)); simulTab();
                write(inttohex(dummy->attribs)); simulTab(); writeln(inttohex(dummy->base2));
        }
}
*/

void getIDTInfo()
{
        extern void idtstart;
        idtinfo.offset = &idtstart;
        idtinfo.items = 0;
        idtinfo.datasel = SYS_DATA_SEL;
}

/*
void printIDT(unsigned from, unsigned to)
{
        unsigned i;
        GATE *dummy;
        if(from > to) return;
        write("IDT.OFFSET: "); write(inttohex((DWORD)gdtinfo.offset)); simulTab(); write("IDT.ITEMS: "); writeln(inttohex(gdtinfo.items));
        for(i = from; i <= to; i++) {
                dummy = gdtinfo.offset + 8 * i;
                write(inttohex(dummy->offset0)); simulTab(); write(inttohex(dummy->codesel)); simulTab();
                write(inttohex(dummy->reserved)); simulTab(); write(inttohex(dummy->access)); simulTab();
                writeln(inttohex(dummy->offset1));
        }
}
*/

void initIDT()
{
        WORD i;
        getIDTInfo();
        for(i = 0; i <= 255; i++) setInterruptGate(i, SYS_CODE_SEL, (DWORD)&int00isr, INTGATE | PRESENT);
        setInterruptGate(IRQ06, SYS_CODE_SEL, (DWORD)&int38isr, INTGATE | PRESENT);
        setInterruptGate(IRQ00, SYS_CODE_SEL, (DWORD)&int32isr, INTGATE | PRESENT);
}

void setInterruptGate(BYTE intno, WORD cs, DWORD off, BYTE type)
{
        GATE *dummy = idtinfo.offset + intno*8;
        dummy->offset0 = (WORD)(off & 0xFFFF);
        dummy->codesel = cs;
        dummy->reserved = 0;
        dummy->access = type;
        dummy->offset1 = (WORD)((off >> 16) & 0xFFFF);
}


void initPIC()
{
        remapIRQs();
        outportb(0xA1, 0xEF);
        outportb(0x21, 0xBA);
        outportb(0x20, 0x20);
        asm("sti");
}

asm (
   ".globl _int00       \n"
   "_int00isr:          \n"
   "iret                \n"
);

asm(
        ".globl _int38isr       \n"
        "_int38isr:             \n"
        "call   _FloppyIsr      \n"
        "iret                   \n"
);

asm(
        ".globl _int32isr       \n"
        "_int32isr:             \n"
        "call   _TimerIsr       \n"
        "iret                   \n"
);

void TimerIsr()
{
        extern SCREEN screen;
        BYTE x, y;
        
        x = screen.x;
        y = screen.y;
        gotoXY(screen.resx-8, 0);
        printTime();
        gotoXY(x, y);
        
        TickCounter++;
        
        outportb(0x20, 0x20);
}

void setIrqFired(BYTE irq)
{
        waitForIrqBitmap |= (1 << irq);
}

BYTE getIrqFired(BYTE irq)
{
        if((waitForIrqBitmap & ((WORD)1 << irq)) == ((WORD)1 << irq)) return 1;
        else return 0;
}

void deleteIrqFired(BYTE irq)
{
        waitForIrqBitmap = (1 << irq);
}

void enableInts() { asm("sti"); }
void disableInts() { asm("cli"); }

/*folgende Function ist aus Dark Fibers OS-FAQ und von mir etwas abgewandelt*/

/*
 * void count_memory (void)
 *
 * probes memory above 1mb
 *
 * last mod : 05sep98 - stuart george
 *            08dec98 - ""     ""
 *            21feb99 - removed dummy calls
 *
 */
 
#define UCHAR BYTE
#define USHORT WORD
#define ULONG DWORD
 
WORD count_memory(void)
{
	register ULONG *mem;
	ULONG	mem_count, a;
	USHORT	memkb;
	UCHAR	irq1, irq2;
	ULONG	cr0;

	/* save IRQ's */
	irq1=inportb(0x21);
	irq2=inportb(0xA1);

	/* kill all irq's */
	outportb(0x21, 0xFF);
	outportb(0xA1, 0xFF);

	mem_count=0;
	memkb=0;

	// store a copy of CR0
	__asm__ __volatile("movl %%cr0, %%eax":"=a"(cr0));

	// invalidate the cache
	// write-back and invalidate the cache
	__asm__ __volatile__ ("wbinvd");

	// plug cr0 with just PE/CD/NW
	// cache disable(486+), no-writeback(486+), 32bit mode(386+)
	__asm__ __volatile__("movl %%eax, %%cr0":: "a" (cr0 | 0x00000001 | 0x40000000 | 0x20000000));

	do
	{
		memkb++;
		mem_count+=1024*1024;
		mem=(ULONG*)mem_count;

		a=*mem;

		*mem=0x55AA55AA;

		// the empty asm calls tell gcc not to rely on whats in its registers
		// as saved variables (this gets us around GCC optimisations)
		asm("":::"memory");
		if(*mem!=0x55AA55AA)
			mem_count=0;
		else
		{
			*mem=0xAA55AA55;
			asm("":::"memory");
			if(*mem!=0xAA55AA55)
				mem_count=0;
		}

		asm("":::"memory");
		*mem=a;
	}while(memkb<4096 && mem_count!=0);

	__asm__ __volatile__("movl %%eax, %%cr0":: "a" (cr0));

	outportb(0x21, irq1);
	outportb(0xA1, irq2);
        return memkb;
}

void remapIRQs()                          /*verschiebt die Hardware Interrupts auf Int 32+*/
{
        outportb(0x20, 0x11);
        outportb(0xA0, 0x11);
        
        outportb(0x21, 0x20);             /*Interrupts des 1. PIC auf int 32 legen*/
        outportb(0xA1, 0x28);             /*und des 2. PIC auf 40*/
        
        outportb(0x21, 4);              /*Kaskadierung ber irq2*/
        outportb(0xA1, 2);
        
        outportb(0x21, 1);              /*Intel Umgebung + manuelle Interruptbeendigung*/
        outportb(0xA1, 1);
}

BYTE BcdToDec(BYTE bcd)
{
        return ((10*((bcd >> 4) & 0xF) + (bcd &0xF)));
}

void printTime()
{
        BYTE a;
        
        outportb(0x70, 0);
        a = BcdToDec(inportb(0x71));
        if (a != system.time.seconds) {
                system.time.seconds = a;
        
                outportb(0x70, 4);
                a = BcdToDec(inportb(0x71));
                system.time.hours = a;
                if(a < 10) write("0");
                write(inttostr(a));
                write(":");
        
                outportb(0x70, 2);
                a = BcdToDec(inportb(0x71));
                system.time.minutes = a;
                if(a < 10) write("0");
                write(inttostr(a));
                write(":");
        
                a = system.time.seconds;
                if(a < 10) write("0");
                write(inttostr(a));
        }
}
