/*
This file is part of ObjetcsOS.

ObjetcsOS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

ObjetcsOS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ObjectsOS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _oos_include_system_h
#define _oos_include_system_h

#include "types.h"

#define SYS_CODE_SEL    8
#define SYS_DATA_SEL    16
#define SYS_SCREEN_SEL  24
#define SYS_LINEAR_SEL  40

#define INITIAL_GDT_ITEMS       6       /*inklusive des Nulldescriptors*/

/* Konstante fr type eines DESCRIPTORS*/
#define ACCESSED            0x01
#define DATASEG_NOTREADABLE 0x00
#define DATASEG             0x02
#define DATASEG_RESERVED    0x04
#define DATASEG_EXPANDABLE  0x06                /* "expand-down" Datensegment */
#define CODESEG_NOTREADABLE 0x08
#define CODESEG             0x0A
#define CODESEG_CONFORM_NR  0x0C
#define CODESEG_CONFORM     0x0E
#define SEGMENT             0x10
#define SYSTEM              0x00
#define DPL0                0x00
#define DPL1                0x20
#define DPL2                0x40
#define DPL3                0x60
#define PRESENT             0x80

/* Konstante fr access */
#define GRAN                0x40
#define _80386              0x30
#define AVL                 0x10
    
    
/*Konstanten fr access des GATE*/
    
#define TSS             0x09
#define TSS_ACTIVE      0x0B
#define CALLGATE        0x0C
#define INTGATE         0x0E
#define TRAPGATE        0x0F
#define DPL0            0x00
#define DPL1            0x20
#define DPL2            0x40
#define DPL3            0x60
#define PRESENT         0x80


/*Die IRQs und ihre Interrupts*/
#define IRQ00           0x20
#define IRQ01           0x21
#define IRQ02           0x22
#define IRQ03           0x23
#define IRQ04           0x24
#define IRQ05           0x25
#define IRQ06           0x26
#define IRQ07           0x27

#define IRQ08           0x28
#define IRQ09           0x29
#define IRQ10           0x2A
#define IRQ11           0x2B
#define IRQ12           0x2C
#define IRQ13           0x2D
#define IRQ14           0x2E
#define IRQ15           0x2F



typedef struct {
        WORD size0;
        WORD base0;
        BYTE base1;
        BYTE access;
        BYTE attribs;
        BYTE base2;
} DESCRIPTOR __attribute__ ((packed));

typedef struct {
        void* offset;
        WORD datasel;
        WORD items;
} DTINFO;

typedef struct {
        WORD limit;
        DWORD base;
} DTR __attribute__ ((packed));         /*Descriptor Table Register*/

typedef struct {
        WORD offset0;
        WORD codesel;
        BYTE reserved;
        BYTE access;
        WORD offset1;
} GATE __attribute__ ((packed));

typedef struct {
        BYTE seconds;
        BYTE minutes;
        BYTE hours;
} TIME;

typedef struct {
        DWORD mem;
        TIME time;
} SYSTEM_INFO;



void outportb(WORD port, BYTE value);
unsigned char inportb(WORD port);
void getGDTInfo();
void printGDT(unsigned from, unsigned to);
void getIDTInfo();
void printIDT(unsigned from, unsigned to);
void setInterruptGate(BYTE intno, WORD cs, DWORD off, BYTE type);
void initPIC();
void initIDT();

void enableInts();
void disableInts();
void enableA20();

void int00isr();
void int32isr();
void int38isr();

void TimerIsr();

WORD count_memory(void);
void remapIRQs();

void setIrqFired(BYTE irq);
BYTE getIrqFired(BYTE irq);
void deleteIrqFired(BYTE irq);
BYTE BcdToDec(BYTE bcd);
void printTime();

#endif
